# -*- coding: utf-8 -*-
#
#  File: account_move_lines.py
#  Module: l10n_ch_vat_on_payment
#
#  Created by cyp@open-net.ch
#
#  Copyright (c) 2010 Open-Net Ltd. All rights reserved.
##############################################################################
#	
#	OpenERP, Open Source Management Solution
#	Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU Affero General Public License as
#	published by the Free Software Foundation, either version 3 of the
#	License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU Affero General Public License for more details.
#
#	You should have received a copy of the GNU Affero General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.	 
#
##############################################################################

import netsvc
from osv import fields, osv
from tools.translate import _
import time

class account_move_line(osv.osv):
	_inherit = 'account.move.line'

	_columns = {
		'state': fields.selection([('draft','Unbalanced'), ('tva','TVA'), ('valid','Valid')], 'State', readonly=True,
									help='When new move line is created the state will be \'Draft\'.\n* When all the payments are done it will be in \'Valid\' state.'),
	}
	
	def _query_get(self, cr, uid, obj='l', context=None):
		handle_tva_state = context.get('handle_tva_state', False)
		fiscalyear_obj = self.pool.get('account.fiscalyear')
		fiscalperiod_obj = self.pool.get('account.period')
		account_obj = self.pool.get('account.account')
		fiscalyear_ids = []
		if context is None:
			context = {}
		initial_bal = context.get('initial_bal', False)
		company_clause = " "
		if context.get('company_id', False):
			company_clause = " AND " +obj+".company_id = %s" % context.get('company_id', False)
		if not context.get('fiscalyear', False):
			if context.get('all_fiscalyear', False):
				#this option is needed by the aged balance report because otherwise, if we search only the draft ones, an open invoice of a closed fiscalyear won't be displayed
				fiscalyear_ids = fiscalyear_obj.search(cr, uid, [])
			else:
				fiscalyear_ids = fiscalyear_obj.search(cr, uid, [('state', '=', 'draft')])
		else:
			#for initial balance as well as for normal query, we check only the selected FY because the best practice is to generate the FY opening entries
			fiscalyear_ids = [context['fiscalyear']]

		fiscalyear_clause = (','.join([str(x) for x in fiscalyear_ids])) or '0'
		state = context.get('state', False)
		where_move_state = ''
		where_move_lines_by_date = ''

		if context.get('date_from', False) and context.get('date_to', False):
			if initial_bal:
				where_move_lines_by_date = " AND " +obj+".move_id IN (SELECT id FROM account_move WHERE date < '" +context['date_from']+"')"
			else:
				where_move_lines_by_date = " AND " +obj+".move_id IN (SELECT id FROM account_move WHERE date >= '" +context['date_from']+"' AND date <= '"+context['date_to']+"')"

		if state:
			if state.lower() not in ['all']:
				where_move_state= " AND "+obj+".move_id IN (SELECT id FROM account_move WHERE account_move.state = '"+state+"')"

		if context.get('period_from', False) and context.get('period_to', False) and not context.get('periods', False):
			if initial_bal:
				period_company_id = fiscalperiod_obj.browse(cr, uid, context['period_from'], context=context).company_id.id
				first_period = fiscalperiod_obj.search(cr, uid, [('company_id', '=', period_company_id)], order='date_start', limit=1)[0]
				context['periods'] = fiscalperiod_obj.build_ctx_periods(cr, uid, first_period, context['period_from'])
			else:
				context['periods'] = fiscalperiod_obj.build_ctx_periods(cr, uid, context['period_from'], context['period_to'])
		if context.get('periods', False):
			if initial_bal:
				query = obj+(handle_tva_state and ".state = 'tva'" or ".state = 'valid'")+" AND "+obj+".period_id IN (SELECT id FROM account_period WHERE fiscalyear_id IN (%s)) %s %s" % (fiscalyear_clause, where_move_state, where_move_lines_by_date)
				period_ids = fiscalperiod_obj.search(cr, uid, [('id', 'in', context['periods'])], order='date_start', limit=1)
				if period_ids and period_ids[0]:
					first_period = fiscalperiod_obj.browse(cr, uid, period_ids[0], context=context)
					# Find the old periods where date start of those periods less then Start period
					periods = fiscalperiod_obj.search(cr, uid, [('date_start', '<', first_period.date_start)])
					periods = ','.join([str(x) for x in periods])
					if periods:
						query = obj+".state = 'valid' AND "+obj+".period_id IN (SELECT id FROM account_period WHERE fiscalyear_id IN (%s) AND id IN (%s)) %s %s" % (fiscalyear_clause, periods, where_move_state, where_move_lines_by_date)
			else:
				ids = ','.join([str(x) for x in context['periods']])
				query = obj+(handle_tva_state and ".state = 'tva'" or ".state = 'valid'")+" AND "+obj+".period_id IN (SELECT id FROM account_period WHERE fiscalyear_id IN (%s) AND id IN (%s)) %s %s" % (fiscalyear_clause, ids, where_move_state, where_move_lines_by_date)
		else:
			query = obj+(handle_tva_state and ".state = 'tva'" or ".state = 'valid'")+" AND "+obj+".period_id IN (SELECT id FROM account_period WHERE fiscalyear_id IN (%s)) %s %s" % (fiscalyear_clause, where_move_state, where_move_lines_by_date)

		if context.get('journal_ids', False):
			query += ' AND '+obj+'.journal_id IN (%s)' % ','.join(map(str, context['journal_ids']))

		if context.get('chart_account_id', False):
			child_ids = account_obj._get_children_and_consol(cr, uid, [context['chart_account_id']], context=context)
			query += ' AND '+obj+'.account_id IN (%s)' % ','.join(map(str, child_ids))

		query += company_clause
		return query

	def reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False, writeoff_period_id=False, writeoff_journal_id=False, context=None):
		account_obj = self.pool.get('account.account')
		move_obj = self.pool.get('account.move')
		move_rec_obj = self.pool.get('account.move.reconcile')
		partner_obj = self.pool.get('res.partner')
		currency_obj = self.pool.get('res.currency')
		lines = self.browse(cr, uid, ids, context=context)
		unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
		credit = debit = 0.0
		currency = 0.0
		account_id = False
		partner_id = False
		if context is None:
			context = {}
		company_list = []
		states_to_update = []
		for line in self.browse(cr, uid, ids, context=context):
			if company_list and not line.company_id.id in company_list:
				raise osv.except_osv(_('Warning !'), _('To reconcile the entries company should be the same for all entries'))
			company_list.append(line.company_id.id)
		for line in unrec_lines:
			if line.state not in ['tva','valid']:
				raise osv.except_osv(_('Error'),
						_('Entry "%s" is not valid !') % line.name)
			credit += line['credit']
			debit += line['debit']
			currency += line['amount_currency'] or 0.0
			account_id = line['account_id']['id']
			partner_id = (line['partner_id'] and line['partner_id']['id']) or False
			states_to_update.append(line.id)
		writeoff = debit - credit

		# Ifdate_p in context => take this date
		if context.has_key('date_p') and context['date_p']:
			date=context['date_p']
		else:
			date = time.strftime('%Y-%m-%d')

		cr.execute('SELECT account_id, reconcile_id '\
				   'FROM account_move_line '\
				   'WHERE id IN %s '\
				   'GROUP BY account_id,reconcile_id',
				   (tuple(ids), ))
		r = cr.fetchall()
		#TODO: move this check to a constraint in the account_move_reconcile object
		if (len(r) != 1) and not context.get('fy_closing', False):
			raise osv.except_osv(_('Error'), _('Entries are not of the same account or already reconciled ! '))
		if not unrec_lines:
			raise osv.except_osv(_('Error'), _('Entry is already reconciled'))
		account = account_obj.browse(cr, uid, account_id, context=context)
		if not context.get('fy_closing', False) and not account.reconcile:
			raise osv.except_osv(_('Error'), _('The account is not defined to be reconciled !'))
		if r[0][1] != None:
			raise osv.except_osv(_('Error'), _('Some entries are already reconciled !'))

		if (not currency_obj.is_zero(cr, uid, account.company_id.currency_id, writeoff)) or \
		   (account.currency_id and (not currency_obj.is_zero(cr, uid, account.currency_id, currency))):
			if not writeoff_acc_id:
				raise osv.except_osv(_('Warning'), _('You have to provide an account for the write off entry !'))
			if writeoff > 0:
				debit = writeoff
				credit = 0.0
				self_credit = writeoff
				self_debit = 0.0
			else:
				debit = 0.0
				credit = -writeoff
				self_credit = 0.0
				self_debit = -writeoff
			# If comment exist in context, take it
			if 'comment' in context and context['comment']:
				libelle = context['comment']
			else:
				libelle = _('Write-Off')

			cur_obj = self.pool.get('res.currency')
			cur_id = False
			amount_currency_writeoff = 0.0
			if context.get('company_currency_id',False) != context.get('currency_id',False):
				cur_id = context.get('currency_id',False)
				for line in unrec_lines:
					if line.currency_id and line.currency_id.id == context.get('currency_id',False):
						amount_currency_writeoff += line.amount_currency
					else:
						tmp_amount = cur_obj.compute(cr, uid, line.account_id.company_id.currency_id.id, context.get('currency_id',False), abs(line.debit-line.credit), context={'date': line.date})
						amount_currency_writeoff += (line.debit > 0) and tmp_amount or -tmp_amount

			writeoff_lines = [
				(0, 0, {
					'name': libelle,
					'debit': self_debit,
					'credit': self_credit,
					'account_id': account_id,
					'date': date,
					'partner_id': partner_id,
					'currency_id': cur_id or (account.currency_id.id or False),
					'amount_currency': amount_currency_writeoff and -1 * amount_currency_writeoff or (account.currency_id.id and -1 * currency or 0.0)
				}),
				(0, 0, {
					'name': libelle,
					'debit': debit,
					'credit': credit,
					'account_id': writeoff_acc_id,
					'analytic_account_id': context.get('analytic_id', False),
					'date': date,
					'partner_id': partner_id,
					'currency_id': cur_id or (account.currency_id.id or False),
					'amount_currency': amount_currency_writeoff and amount_currency_writeoff or (account.currency_id.id and currency or 0.0)
				})
			]

			writeoff_move_id = move_obj.create(cr, uid, {
				'period_id': writeoff_period_id,
				'journal_id': writeoff_journal_id,
				'date':date,
				'state': 'draft',
				'line_id': writeoff_lines
			})

			writeoff_line_ids = self.search(cr, uid, [('move_id', '=', writeoff_move_id), ('account_id', '=', account_id)])
			if account_id == writeoff_acc_id:
				writeoff_line_ids = [writeoff_line_ids[1]]
			ids += writeoff_line_ids

		self.write( cr, uid, states_to_update, {'state':'valid'}, context=context )
		r_id = move_rec_obj.create(cr, uid, {
			'type': type,
			'line_id': map(lambda x: (4, x, False), ids),
			'line_partial_ids': map(lambda x: (3, x, False), ids)
		})
		wf_service = netsvc.LocalService("workflow")
		# the id of the move.reconcile is written in the move.line (self) by the create method above
		# because of the way the line_id are defined: (4, x, False)
		for id in ids:
			wf_service.trg_trigger(uid, 'account.move.line', id, cr)

		if lines and lines[0]:
			partner_id = lines[0].partner_id and lines[0].partner_id.id or False
			if partner_id and context and context.get('stop_reconcile', False):
				partner_obj.write(cr, uid, [partner_id], {'last_reconciliation_date': time.strftime('%Y-%m-%d %H:%M:%S')})
		return r_id

account_move_line()
