# -*- coding: utf-8 -*-
#
#  File: banks.py
#  Module: l10n_ch_vat_on_payment
#
#  Created by cyp@open-net.ch
#
#  Copyright (c) 2010 Open-Net Ltd. All rights reserved.
##############################################################################
#	
#	OpenERP, Open Source Management Solution
#	Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU Affero General Public License as
#	published by the Free Software Foundation, either version 3 of the
#	License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU Affero General Public License for more details.
#
#	You should have received a copy of the GNU Affero General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.	 
#
##############################################################################

import netsvc
from osv import fields, osv
import decimal_precision as dp

class account_bank_statement(osv.osv):
	_inherit = "account.bank.statement"

	def create_move_from_st_line(self, cr, uid, st_line_id, company_currency_id, st_line_number, context=None):
		if context is None:
			context = {}
		res_currency_obj = self.pool.get('res.currency')
		account_move_obj = self.pool.get('account.move')
		account_move_line_obj = self.pool.get('account.move.line')
		account_bank_statement_line_obj = self.pool.get('account.bank.statement.line')
		st_line = account_bank_statement_line_obj.browse(cr, uid, st_line_id, context=context)
		st = st_line.statement_id

		context.update({'date': st_line.date})

		move_id = account_move_obj.create(cr, uid, {
			'journal_id': st.journal_id.id,
			'period_id': st.period_id.id,
			'date': st_line.date,
			'name': st_line_number,
		}, context=context)
		account_bank_statement_line_obj.write(cr, uid, [st_line.id], {
			'move_ids': [(4, move_id, False)]
		})

		torec = []
		if st_line.amount >= 0:
			account_id = st.journal_id.default_credit_account_id.id
		else:
			account_id = st.journal_id.default_debit_account_id.id

		acc_cur = ((st_line.amount<=0) and st.journal_id.default_debit_account_id) or st_line.account_id
		context.update({
				'res.currency.compute.account': acc_cur,
			})
		amount = res_currency_obj.compute(cr, uid, st.currency.id,
				company_currency_id, st_line.amount, context=context)

		val = {
			'name': st_line.name,
			'date': st_line.date,
			'ref': st_line.ref,
			'move_id': move_id,
			'partner_id': ((st_line.partner_id) and st_line.partner_id.id) or False,
			'account_id': (st_line.account_id) and st_line.account_id.id,
			'credit': ((amount>0) and amount) or 0.0,
			'debit': ((amount<0) and -amount) or 0.0,
			'statement_id': st.id,
			'journal_id': st.journal_id.id,
			'period_id': st.period_id.id,
			'currency_id': st.currency.id,
			'analytic_account_id': st_line.analytic_account_id and st_line.analytic_account_id.id or False
		}

		if st.currency.id <> company_currency_id:
			amount_cur = res_currency_obj.compute(cr, uid, company_currency_id,
						st.currency.id, amount, context=context)
			val['amount_currency'] = -amount_cur

		if st_line.account_id and st_line.account_id.currency_id and st_line.account_id.currency_id.id <> company_currency_id:
			val['currency_id'] = st_line.account_id.currency_id.id
			amount_cur = res_currency_obj.compute(cr, uid, company_currency_id,
					st_line.account_id.currency_id.id, amount, context=context)
			val['amount_currency'] = -amount_cur

		move_line_id = account_move_line_obj.create(cr, uid, val, context=context)
		torec.append(move_line_id)

		# Fill the secondary amount/currency
		# if currency is not the same than the company
		amount_currency = False
		currency_id = False
		if st.currency.id <> company_currency_id:
			amount_currency = st_line.amount
			currency_id = st.currency.id
		account_move_line_obj.create(cr, uid, {
			'name': st_line.name,
			'date': st_line.date,
			'ref': st_line.ref,
			'move_id': move_id,
			'partner_id': ((st_line.partner_id) and st_line.partner_id.id) or False,
			'account_id': account_id,
			'credit': ((amount < 0) and -amount) or 0.0,
			'debit': ((amount > 0) and amount) or 0.0,
			'statement_id': st.id,
			'journal_id': st.journal_id.id,
			'period_id': st.period_id.id,
			'amount_currency': amount_currency,
			'currency_id': currency_id,
			}, context=context)

		for line in account_move_line_obj.browse(cr, uid, [x.id for x in
				account_move_obj.browse(cr, uid, move_id,
					context=context).line_id],
				context=context):
			if line.state not in ['tva','valid']:
				raise osv.except_osv(_('Error !'),
						_('Journal Item "%s" is not valid') % line.name)

		# Bank statements will not consider boolean on journal entry_posted
		account_move_obj.post(cr, uid, [move_id], context=context)
		return move_id

	def button_confirm_bank(self, cr, uid, ids, context=None):
		obj_seq = self.pool.get('ir.sequence')
		if context is None:
			context = {}

		for st in self.browse(cr, uid, ids, context=context):
			j_type = st.journal_id.type
			company_currency_id = st.journal_id.company_id.currency_id.id
			if not self.check_status_condition(cr, uid, st.state, journal_type=j_type):
				continue

			self.balance_check(cr, uid, st.id, journal_type=j_type, context=context)
			if (not st.journal_id.default_credit_account_id) \
					or (not st.journal_id.default_debit_account_id):
				raise osv.except_osv(_('Configuration Error !'),
						_('Please verify that an account is defined in the journal.'))

			if not st.name == '/':
				st_number = st.name
			else:
				if st.journal_id.sequence_id:
					c = {'fiscalyear_id': st.period_id.fiscalyear_id.id}
					st_number = obj_seq.get_id(cr, uid, st.journal_id.sequence_id.id, context=c)
				else:
					st_number = obj_seq.get(cr, uid, 'account.bank.statement')

			for line in st.move_line_ids:
				if line.state not in ['tva','valid']:
					raise osv.except_osv(_('Error !'),
							_('The account entries lines are not in valid state.'))
			for st_line in st.line_ids:
				if st_line.analytic_account_id:
					if not st.journal_id.analytic_journal_id:
						raise osv.except_osv(_('No Analytic Journal !'),_("You have to define an analytic journal on the '%s' journal!") % (st.journal_id.name,))
				if not st_line.amount:
					continue
				st_line_number = self.get_next_st_line_number(cr, uid, st_number, st_line, context)
				self.create_move_from_st_line(cr, uid, st_line.id, company_currency_id, st_line_number, context)

			self.write(cr, uid, [st.id], {'name': st_number}, context=context)
			self.log(cr, uid, st.id, _('Statement %s is confirmed, journal items are created.') % (st_number,))
		return self.write(cr, uid, ids, {'state':'confirm'}, context=context)

account_bank_statement()
